'************************************************
'* MultiModem Demo Application - MainForm
'* Environment: Visual Basic 2005
'* Programmer: Sam Javner
'* Date: 4/29/2007
'*
'* Summary: Demo Application implementing the use of the
'* MultiTech MultiModem using the MultiModem Class
'*
'* Note: Any visual elements on the form (buttons, labels, etc.)
'* are prefixed with "ux" (user experience) so that they appear
'* together in the Properties dropdown list
'************************************************



Public Class MainForm

    'Variable for instance of MultiModem Class
    Public MM As New MultiModem

    'Variables for timeout for ListMessages
    Private listMessagesLastTime As Single
    Public ListMessagesTimeout As Long


    '************************************************
    '* Private Sub MainForm_Load
    '* Summary:     Occurs on form load, readies the form for use
    '************************************************
    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        'Populate serial port list
        For Each portName As String In My.Computer.Ports.SerialPortNames
            uxSerialPortList.Items.Add(portName)
        Next

        'Select first index as default
        uxSerialPortList.SelectedIndex = 0

        'Enable only the Connect Button
        uxConnectButton.Enabled = True
        uxSerialPortList.Enabled = True
        SetButtons(False)

        'Create column headers for Message List
        CreateColumnHeaders()

        'Set interval for Timers
        CheckMessagesTimer.Interval = 100
        SignalQualityTimer.Interval = 10000

        'Set timeout for ListMessages (in milliseconds)
        ListMessagesTimeout = 2000

    End Sub


    '************************************************
    '* Private Sub ExitToolStripMenuItem_Click
    '* Summary:     Handles File->Exit menu item which closes program
    '************************************************
    Private Sub ExitToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitToolStripMenuItem.Click

        'Disconnect from serial port if connected
        If uxDisconnectButton.Enabled Then uxDisconnectButton_Click(sender, e)

        'Unload the form
        Me.Close()

    End Sub



    '************************************************
    '* Private Sub SynchronizeClockToolStripMenuItem_Click
    '* Summary:     Handles Synchronize Clock menu item which
    '*              synchronizes the wireless modem's clock to the computer's
    '************************************************
    Private Sub SynchronizeClockToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SynchronizeClockToolStripMenuItem.Click

        Dim clock As String

        'Format the time string properly for the SetClock function
        clock = Format(Now, "yy/MM/dd,hh:mm:ss")

        'Set clock and check for success
        If MM.SetClock(clock) Then
            uxStatusBar.Text = "Clock set to: " & clock
        Else
            MsgBox("Error synchronizing clock", vbExclamation, "MultiModem Demo Application")
        End If

    End Sub



    '************************************************
    '* Private Sub InfoToolStripMenuItem_Click
    '* Summary:     Handles Info menu item which displays info from wireless modem
    '************************************************
    Private Sub InfoToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InfoToolStripMenuItem.Click

        Dim subscriber As String = String.Empty, manufacturer As String = String.Empty, model As String = String.Empty
        Dim revision As String = String.Empty, serial As String = String.Empty, clock As String = String.Empty

        'Retrieve information, check for errors

        If Not MM.GetSubscriberNumber(subscriber) Then
            MsgBox("Error retrieving Subscriber Number", vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        If Not MM.GetManufacturerID(manufacturer) Then
            MsgBox("Error retrieving Manufacturer ID", vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        If Not MM.GetModelID(model) Then
            MsgBox("Error retrieving Model ID", vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        If Not MM.GetRevisionID(revision) Then
            MsgBox("Error retrieving Revision ID", vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        If Not MM.GetSerialNumber(serial) Then
            MsgBox("Error retrieving Serial Number", vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        If Not MM.GetClock(clock) Then
            MsgBox("Error retrieving Clock", vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        'Display information in a message box

        MsgBox("Subscriber ID: " & FormatPhoneNumber(subscriber) & vbCrLf & _
                "Manufacturer ID: " & manufacturer & vbCrLf & _
                "Model ID: " & model & vbCrLf & _
                "Revision ID: " & revision & vbCrLf & _
                "Serial Number: " & serial & vbCrLf & _
                "Date/Time: " & clock _
                , vbOKOnly, "MultiModem Demo Application")

    End Sub



    '************************************************
    '* Private Sub uxConnectButton_Click
    '* Summary:     Handles Connect Button, connects to serial port
    '************************************************
    Private Sub uxConnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles uxConnectButton.Click

        Dim port As Integer

        port = CInt(Mid(uxSerialPortList.Text, 4))

        'Try connecting to selected port, display message if error connecting
        If Not MM.Connect(port) Then
            MsgBox("Error connecting to port " & port, vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        'Disable echoing of AT Commands should be done immediately after connecting
        'Any other MultiModem function calls expect echoing to be disabled
        If Not MM.SetEcho(False) Then
            MsgBox("Error disabling Echo", vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        'Set New Message Indication = 1
        'If we don't do this, CheckMessages might not work
        If Not MM.SetNewMessageIndication(1) Then
            MsgBox("Error setting New Message Indication", vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        'Disable connect button
        uxConnectButton.Enabled = False
        uxSerialPortList.Enabled = False

        'Display message
        uxStatusBar.Text = "Connected to port " & port

        'Make sure buttons are disabled
        'Don't want to be issuing commands while listing messages
        SetButtons(False)

        'List Messages
        ListMessages()

        'Done, enable buttons
        SetButtons(True)

        'Enable Timers
        CheckMessagesTimer.Enabled = True
        SignalQualityTimer.Enabled = True

    End Sub



    '************************************************
    '* Private Sub uxCreateButton_Click
    '* Summary:     Handles Create Button, opens form for a new text message
    '************************************************
    Private Sub uxCreateButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles uxCreateButton.Click

        Dim subscriber As String

        Dim cmf As New CreateMessageForm

        'Get subscriber number and update on CreateMessageForm
        MM.GetSubscriberNumber(subscriber)
        cmf.uxFromValue.Text = FormatPhoneNumber(subscriber)

        cmf.uxSMS.Text = String.Empty

        'Display CreateMessageForm
        cmf.ShowDialog()

    End Sub



    '************************************************
    '* Private Sub uxDeleteButton_Click
    '* Summary:     Handles Delete Button, deletes a received text message
    '************************************************
    Private Sub uxDeleteButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles uxDeleteButton.Click

        Dim index As Integer

        If uxMessageList.SelectedItems.Count = 0 Then Exit Sub
        index = uxMessageList.SelectedItems(0).Text

        If MM.DeleteMessage(index) Then uxMessageList.SelectedItems(0).Remove()

    End Sub



    '************************************************
    '* Private Sub uxDisconnectButton_Click
    '* Summary:     Handles Disconnect Button, disconnects from serial port
    '************************************************
    Private Sub uxDisconnectButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles uxDisconnectButton.Click

        Dim port As Integer

        port = CInt(Mid(uxSerialPortList.Text, 4))

        'Disable Timers
        CheckMessagesTimer.Enabled = False
        SignalQualityTimer.Enabled = False
        uxSignalLabel.Text = String.Empty
        uxSignalBox.Image = Nothing

        'Try disconnecting, display message if error disconnecting
        If Not MM.Disconnect Then
            MsgBox("Error disconnecting from port " & port, vbExclamation, "MultiModem Demo Application")
            Exit Sub
        End If

        'Enable connect button and disable other buttons
        uxConnectButton.Enabled = True
        uxSerialPortList.Enabled = True
        SetButtons(False)

        'Display message
        uxStatusBar.Text = "Disconnected from port " & port

        'Remove messages from list
        Dim i As Integer
        For i = uxMessageList.Items.Count - 1 To 0 Step -1
            uxMessageList.Items(i).Remove()
        Next i

    End Sub



    '************************************************
    '* Private Sub uxRefreshButton_Click
    '* Summary:     Handles Refresh Button, refreshes list of text messages
    '************************************************
    Private Sub uxRefreshButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles uxRefreshButton.Click

        'Make sure buttons are disabled
        'Don't want to be issuing commands while listing messages
        SetButtons(False)

        'List Messages
        ListMessages()

        'Done, enable buttons
        SetButtons(True)

    End Sub



    '************************************************
    '* Private Sub uxViewButton_Click
    '* Summary:     Handles View Button, opens form to view a message in more detail
    '************************************************
    Private Sub uxViewButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles uxViewButton.Click

        Dim subscriber As String
        Dim unread As Boolean, number As String, timestamp As String, msg As String, msghex As String

        Dim vmf As New ViewMessageForm

        'Update ViewMessageForm with subscriber number
        MM.GetSubscriberNumber(subscriber)
        vmf.uxToValue.Text = FormatPhoneNumber(subscriber)

        'Read the currently selected message as text and hex
        MM.SetCharacterSet("PCCP437")
        MM.ReadMessage(uxMessageList.SelectedItems(0).Text, unread, number, timestamp, msg)
        MM.SetCharacterSet("HEX")
        MM.ReadMessage(uxMessageList.SelectedItems(0).Text, unread, number, timestamp, msghex)

        'Update ViewMessageForm with info from ReadMessage
        vmf.uxFromValue.Text = FormatPhoneNumber(number)
        vmf.uxSMS.Text = msg
        vmf.uxSMSHex.Text = msghex

        'Show ViewMessageForm
        vmf.ShowDialog()

    End Sub



    '************************************************
    '* Private Sub CheckMessagesTimer_Tick
    '* Summary:     Check the message queue for any incoming messages
    '************************************************
    Private Sub CheckMessagesTimer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles CheckMessagesTimer.Tick

        Dim index As Integer, unread As Boolean, number As String = String.Empty, timestamp As String = String.Empty, msg As String = String.Empty

        'Check the incoming message queue
        'If there is a new message, the index of the message will be stored in index
        If MM.CheckMessages(index) Then

            'Set to text mode and read message
            MM.SetCharacterSet("PCCP437")
            MM.ReadMessage(index, unread, number, timestamp, msg)

            'Add message to Message List
            UpdateMessageList(index, unread, FormatPhoneNumber(number), timestamp, msg)

            'Auto reply if checked
            If uxAutoReply.Checked = True Then MM.SendMessage(number, "Message Received Successfully at " & Now & ": " & Microsoft.VisualBasic.Left(msg, 10) & "...")

        End If

    End Sub



    '************************************************
    '* Private Sub CheckListMessagesTimer_Tick
    '* Summary:     Check the list message queue for messages
    '*              Timer is enabled from ListMessages
    '************************************************
    Private Sub CheckListMessagesTimer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles CheckListMessagesTimer.Tick

        Dim index As Integer, unread As Boolean, number As String = String.Empty, timestamp As String = String.Empty, msg As String = String.Empty

        'Add message to Message List
        If MM.CheckListMessages(index, unread, number, timestamp, msg) Then

            listMessagesLastTime = Microsoft.VisualBasic.Timer
            UpdateMessageList(index, unread, FormatPhoneNumber(number), timestamp, msg)

        End If

        'Assume there are no more messages if it has been
        'ListMessageTimeout milliseconds since the last message
        If (listMessagesLastTime + (ListMessagesTimeout / 1000)) < Microsoft.VisualBasic.Timer Then

            CheckListMessagesTimer.Enabled = False

        End If

    End Sub



    '************************************************
    '* Private Sub SignalQualityTimer_Tick
    '* Summary:     Check the list message queue for messages
    '*              Timer is enabled from ListMessages
    '************************************************
    Private Sub SignalQualityTimer_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles SignalQualityTimer.Tick

        Dim strength As Integer
        Dim color As Long

        If Not MM.GetSignalQuality(strength) Then Exit Sub

        'Clear picture box
        uxSignalBox.Image = Nothing

        If strength = 99 Then
            uxSignalLabel.Text = "NS"        'No Signal
            Exit Sub
        Else
            uxSignalLabel.Text = strength
        End If

        'Pick a color based on signal strength
        color = RGB(255, 0, 0)                          'Red
        If strength > 6 Then color = RGB(255, 127, 0) 'Orange
        If strength > 12 Then color = RGB(0, 255, 0) 'Green

        'Display signal bars
        'If strength > 0 Then uxSignalBox.Line (1, 16)-(3, 13), color, BF
        'If strength > 6 Then uxSignalBox.Line (5, 16)-(7, 10), color, BF
        'If strength > 12 Then uxSignalBox.Line (9, 16)-(11, 7), color, BF
        'If strength > 18 Then uxSignalBox.Line (13, 16)-(15, 4), color, BF
        'If strength > 24 Then uxSignalBox.Line (17, 16)-(19, 1), color, BF

    End Sub



    '************************************************
    '* Private Sub CreateColumnHeaders()
    '* Summary:     Creates column headers for the Message List
    '************************************************
    Private Sub CreateColumnHeaders()

        'Clear column headers
        uxMessageList.Columns.Clear()

        'Create column headers
        uxMessageList.Columns.Add("Index", 50)
        uxMessageList.Columns.Add("Status", 60)
        uxMessageList.Columns.Add("From", 100)
        uxMessageList.Columns.Add("Sent", 120)
        uxMessageList.Columns.Add("Message", 250)

    End Sub



    '************************************************
    '* Private Function FormatPhoneNumber(number As String)
    '* Summary:     Formats a 10 or 11 digit phone number
    '* Parameters:  number - phone number to format
    '************************************************
    Private Function FormatPhoneNumber(ByVal number As String)

        If Len(number) = 10 Then
            FormatPhoneNumber = Mid(number, 1, 3) & "-" & _
                                Mid(number, 4, 3) & "-" & _
                                Mid(number, 7, 4)
        ElseIf Len(number) = 11 Then
            FormatPhoneNumber = Mid(number, 1, 1) & "-" & _
                                Mid(number, 2, 3) & "-" & _
                                Mid(number, 5, 3) & "-" & _
                                Mid(number, 8, 4)
        Else
            FormatPhoneNumber = number
        End If

    End Function



    '************************************************
    '* Private Sub ListMessages
    '* Summary:     Lists saved messages
    '************************************************
    Private Sub ListMessages()

        'Disable Check Messages Timer
        CheckMessagesTimer.Enabled = False
        SignalQualityTimer.Enabled = False

        'Set character set to PCCP437 (text, not Hex) and List messages
        MM.SetCharacterSet("PCCP437")
        MM.ListMessages()

        listMessagesLastTime = Microsoft.VisualBasic.Timer

        'Enable timer to read messages from ListMessages
        CheckListMessagesTimer.Interval = 100
        CheckListMessagesTimer.Enabled = True

        'Wait for messages
        While CheckListMessagesTimer.Enabled = True
            Application.DoEvents()
        End While

        'Enable Check Messages Timer
        CheckMessagesTimer.Enabled = True
        SignalQualityTimer.Enabled = True

    End Sub



    '************************************************
    '* Private Sub SetButtons(enable As Boolean)
    '* Summary:     Grays out ore enables buttons
    '* Parameters:  enable - True to enable, False to disable
    '************************************************
    Private Sub SetButtons(ByVal enable As Boolean)

        If enable Then

            uxDisconnectButton.Enabled = True
            uxRefreshButton.Enabled = True
            uxViewButton.Enabled = True
            uxCreateButton.Enabled = True
            uxDeleteButton.Enabled = True
            SynchronizeClockToolStripMenuItem.Enabled = True
            InfoToolStripMenuItem.Enabled = True

        Else

            uxDisconnectButton.Enabled = False
            uxRefreshButton.Enabled = False
            uxViewButton.Enabled = False
            uxCreateButton.Enabled = False
            uxDeleteButton.Enabled = False
            SynchronizeClockToolStripMenuItem.Enabled = False
            InfoToolStripMenuItem.Enabled = False

        End If

    End Sub



    '************************************************
    '* Private Sub UpdateMessageList(index As Integer, unread As Boolean,
    '*             number As String, timestamp As String, msg As String)
    '* Summary:     Adds a message to the Message List
    '* Parameters:  index - index of message
    '*              unread - whether or not message has been read
    '*              timestamp - time of message
    '*              msg - contents of message
    '************************************************
    Private Sub UpdateMessageList(ByVal index As Integer, ByVal unread As Boolean, ByVal number As String, ByVal timestamp As String, ByVal msg As String)

        Dim li As ListViewItem
        Dim i As Integer
        Dim found As Boolean

        found = False

        'Update message if already in list
        For i = 0 To uxMessageList.Items.Count - 1

            If uxMessageList.Items(i).SubItems(0).Text = index Then

                uxStatusBar.Text = "Updating Index " & index & "..."

                uxMessageList.Items(i).SubItems(1).Text = IIf(unread, "Unread", "Read")
                uxMessageList.Items(i).SubItems(2).Text = number
                uxMessageList.Items(i).SubItems(3).Text = timestamp
                uxMessageList.Items(i).SubItems(4).Text = msg

                found = True

                Exit For

            End If

        Next

        'Add to list if not already in list
        If Not found Then

            uxStatusBar.Text = "Updating Index " & index & "..."

            li = New ListViewItem(index)

            li.SubItems.Add(IIf(unread, "Unread", "Read"))
            li.SubItems.Add(number)
            li.SubItems.Add(timestamp)
            li.SubItems.Add(msg)

            uxMessageList.Items.Add(li)

        End If


    End Sub



End Class
